syntax = "proto3";

message ConLMetrics {
  string name = 1;
  float value = 2;
}

message ConLStats {
  string src = 1;
  int32 round = 2;
  repeated ConLMetrics metrics = 3;
}

message ConLParams {
  string op = 1;
  bytes params = 2;

  ConLStats stats = 4;
}
