# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: conl.proto
# Protobuf Python Version: 6.30.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    2,
    '',
    'conl.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\nconl.proto\"*\n\x0b\x43onLMetrics\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x02\"F\n\tConLStats\x12\x0b\n\x03src\x18\x01 \x01(\t\x12\r\n\x05round\x18\x02 \x01(\x05\x12\x1d\n\x07metrics\x18\x03 \x03(\x0b\x32\x0c.ConLMetrics\"C\n\nConLParams\x12\n\n\x02op\x18\x01 \x01(\t\x12\x0e\n\x06params\x18\x02 \x01(\x0c\x12\x19\n\x05stats\x18\x04 \x01(\x0b\x32\n.ConLStatsb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'conl_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_CONLMETRICS']._serialized_start=14
  _globals['_CONLMETRICS']._serialized_end=56
  _globals['_CONLSTATS']._serialized_start=58
  _globals['_CONLSTATS']._serialized_end=128
  _globals['_CONLPARAMS']._serialized_start=130
  _globals['_CONLPARAMS']._serialized_end=197
# @@protoc_insertion_point(module_scope)
